@@ -123,6 +123,37 @@ Deno.test({
   },
 });
 
+Deno.test({
+  name: "createGraph() - resolveTypes",
+  async fn() {
+    const graph = await createGraph(
+      "file:///a.js",
+      {
+        load(specifier) {
+          if (specifier === "file:///a.js") {
+            return Promise.resolve({
+              specifier: "file:///a.js",
+              content: `export const a = "a";`,
+            });
+          } else {
+            return Promise.resolve({
+              specifier: "file:///a.d.ts",
+              content: `export const a: "a";`,
+            });
+          }
+        },
+        resolveTypes(specifier) {
+          assertEquals(specifier, "file:///a.js");
+          return {
+            types: "file:///a.d.ts",
+          };
+        },
+      },
+    );
+    assertEquals(graph.modules.length, 2);
+  },
+});
+
 Deno.test({
   name: "load() - remote module",
   async fn() {
@@ -169,7 +200,7 @@ Deno.test({
         "specifier": "./a.ts",
         "code": {
           "specifier": "file:///a/a.ts",
-          "span": {
+          "range": {
             "start": { "line": 2, "character": 26 },
             "end": { "line": 2, "character": 34 },
           },
@@ -178,7 +209,7 @@ Deno.test({
         "specifier": "./b.ts",
         "code": {
           "specifier": "file:///a/b.ts",
-          "span": {
+          "range": {
             "start": { "line": 3, "character": 27 },
             "end": { "line": 3, "character": 35 },
           },
@@ -187,7 +218,7 @@ Deno.test({
         "specifier": "./c.ts",
         "code": {
           "specifier": "file:///a/c.ts",
-          "span": {
+          "range": {
             "start": { "line": 4, "character": 26 },
             "end": { "line": 4, "character": 34 },
           },
@@ -196,19 +227,20 @@ Deno.test({
         "specifier": "./d.ts",
         "code": {
           "specifier": "file:///a/d.ts",
-          "span": {
+          "range": {
             "start": { "line": 5, "character": 31 },
             "end": { "line": 5, "character": 39 },
           },
         },
+        "isDynamic": true,
       }],
       "typesDependency": {
         "specifier": "./test01.d.ts",
         "dependency": {
           "specifier": "file:///a/test01.d.ts",
-          "span": {
-            "start": { "line": 1, "character": 29 },
-            "end": { "line": 1, "character": 42 },
+          "range": {
+            "start": { "line": 1, "character": 30 },
+            "end": { "line": 1, "character": 43 },
           },
         },
       },
